@@ -15,9 +15,9 @@
 #' @param end Character. End of the time series (only if dateVector is empty)
 #' @param frequency Character. Frequency of the time series (only if dateVector is empty)
 #' @param format Character. Date format (only if dateVector is empty)
-#' @param restriction_matrix Matrix. A matrix containing presupposed entries for matrix B, NA if no restriction is imposed (entries to be estimated)
+#' @param restriction_matrix Matrix or vector. A matrix containing presupposed entries for matrix B, NA if no restriction is imposed (entries to be estimated). Alternatively, a K^2*J (or J*K^2) matrix can be passed, where J is the number of restrictions and K the number of time series. (as suggested in Luetkepohl, 2017, section 5.2.1).
 #' @param max.iter Integer. Number of maximum GLS iterations
-#' @param crit Integer. Critical value for the precision of the GLS estimation
+#' @param crit Numeric. Critical value for the precision of the GLS estimation
 #' @return A list of class "svars" with elements
 #' \item{Lambda}{Estimated unconditional heteroscedasticity matrix \eqn{\Lambda}}
 #' \item{Lambda_SE}{Matrix of standard errors of Lambda}
@@ -30,7 +30,7 @@
 #' \item{iteration}{Number of GLS estimations}
 #' \item{method}{Method applied for identification}
 #' \item{SB}{Structural break (number of observations)}
-#' \item{A_hat}{Estimated VAR paramter via GLS}
+#' \item{A_hat}{Estimated VAR parameter via GLS}
 #' \item{type}{Type of the VAR model, e.g. 'const'}
 #' \item{SBcharacter}{Structural break (date; if provided in function arguments)}
 #' \item{restrictions}{Number of specified restrictions}
@@ -45,8 +45,7 @@
 #' @seealso For alternative identification approaches see \code{\link{id.st}}, \code{\link{id.cvm}}, \code{\link{id.dc}} or \code{\link{id.ngml}}
 #'
 #' @examples
-#' \donttest{
-#' # data contains quartlery observations from 1965Q1 to 2008Q2
+#' #' # data contains quartlery observations from 1965Q1 to 2008Q2
 #' # assumed structural break in 1979Q3
 #' # x = output gap
 #' # pi = inflation
@@ -71,6 +70,11 @@
 #' x2 <- id.cv(v1, SB = 59, restriction_matrix = restMat)
 #' summary(x2)
 #'
+#' # In alternative Form
+#' restMat <- c(rep(0,6), 1, 0,0)
+#' x2 <- id.cv(v1, SB = 59, restriction_matrix = restMat)
+#' summary(x2)
+#'
 #' #Structural brake via Dates
 #' # given that time series vector with dates is available
 #' dateVector = seq(as.Date("1965/1/1"), as.Date("2008/7/1"), "quarter")
@@ -86,7 +90,6 @@
 #' x5 <- id.cv(v1, SB = c(1979, 3))
 #' summary(x5)
 #'
-#' }
 #' @importFrom steadyICA steadyICA
 #' @export
 
@@ -100,70 +103,121 @@
 
 
 id.cv <- function(x, SB, start = NULL, end = NULL, frequency = NULL,
-                        format = NULL, dateVector = NULL, max.iter = 50, crit = 0.001, restriction_matrix = NULL){
+                        format = NULL, dateVector = NULL, max.iter = 50, crit = 0.001,
+                  restriction_matrix = NULL){
+
+
+
 u <- Tob <- p <- k <- residY <- coef_x <- yOut <- type <- y <-  NULL
 get_var_objects(x)
 
-  if(is.numeric(SB)){
+rmOut = restriction_matrix
+restriction_matrix <- get_restriction_matrix(restriction_matrix, k)
+
+restrictions <- length(restriction_matrix[!is.na(restriction_matrix)])
+
+  if (is.numeric(SB)) {
     SBcharacter <- NULL
   }
 
-if(!is.numeric(SB)){
+if (!is.numeric(SB)) {
     SBcharacter <- SB
     SB <- getStructuralBreak(SB = SB, start = start, end = end,
                              frequency = frequency, format = format, dateVector = dateVector, Tob = Tob, p = p)
-  }
-
-    if(length(SB) != 1 & inherits(x$y, "ts")){
-      SBts = SB
-      SB = dim(window(x$y, end = SB))[1]
+} else if(length(SB) != 1 & inherits(x$y, "ts") & length(SB) < 4){
+        SBts = SB
+        SB = dim(window(x$y, end = SB))[1]
       if(frequency(x$y == 4)){
         SBcharacter = paste(SBts[1], " Q", SBts[2], sep = "")
-      }else if(frequency(x$y == 12)){
+     }else if(frequency(x$y == 12)){
         SBcharacter = paste(SBts[1], " M", SBts[2], sep = "")
-      }else if(frequency(x$y == 52)){
+     }else if(frequency(x$y == 52)){
         SBcharacter = paste(SBts[1], " W", SBts[2], sep = "")
-      }else if(frequency(x$y == 365.25)){
+     }else if(frequency(x$y == 365.25)){
         SBcharacter = paste(SBts[1], "-", SBts[2], "-", SBts[3], sep = "")
-      }else{
+     }else{
         SBcharacter = NULL
-      }
-
-    }
+  }
+}
 
+if (length(SB) > 4 & length(SB) != Tob) {
+  stop('Wrong length of SB')
+}
 
+if (length(SB) == Tob) {
+  SB_out <- SB
+  TB <- Tob - sum(SB) + 1
+  resid1 <- u[SB == 0,]
+  resid2 <- u[SB,]
+} else {
+  SB_out <- SB
   TB <- SB - p
+  SB <- rep(0, Tob)
+  SB[TB:Tob] <- 1
+  resid1 <- u[1:TB - 1, ]
+  resid2 <- u[TB:Tob, ]
+}
 
-  resid1 <- u[1:TB-1,]
-  resid2 <- u[TB:Tob,]
-  Sigma_hat1 <- (crossprod(resid1)) / (TB-1)
-  Sigma_hat2 <- (crossprod(resid2)) / (Tob-TB+1)
+Sigma_hat1 <- (crossprod(resid1)) / (TB - 1)
+Sigma_hat2 <- (crossprod(resid2)) / (Tob - TB + 1)
 
-  if(!is.null(restriction_matrix)){
-   resultUnrestricted <- identifyVolatility(x, SB, Tob = Tob, u = u, k = k, y = y, restriction_matrix = NULL,
-                                 Sigma_hat1 = Sigma_hat1, Sigma_hat2 = Sigma_hat2, p = p, TB = TB, SBcharacter,
-                                 max.iter = max.iter, crit = crit, yOut = yOut, type = type)
-    result <- identifyVolatility(x, SB, Tob = Tob, u = u, k = k, y = y, restriction_matrix = restriction_matrix,
-                                           Sigma_hat1 = Sigma_hat1, Sigma_hat2 = Sigma_hat2, p = p, TB = TB, SBcharacter,
-                                 max.iter = max.iter, crit = crit, yOut = yOut, type = type)
+yl <- t(YLagCr(t(y), p))
+yret <- y
+y <- y[, -c(1:p)]
 
-    lRatioTestStatistic = 2 * (resultUnrestricted$Lik - result$Lik)
-    pValue = round(1 - pchisq(lRatioTestStatistic, result$restrictions), 4)
+  if(x$type == 'const'){
+    Z_t <- rbind(rep(1, ncol(yl)), yl)
+  }else if(x$type == 'trend'){
+    Z_t <- rbind(seq(p + 1, ncol(yret)), yl)
+  }else if(x$type == 'both'){
+    Z_t <- rbind(rep(1, ncol(yl)), seq(p + 1, ncol(yret)), yl)
+  }else{
+    Z_t <- yl
+  }
 
-    #result$lRatioTestStatistic = lRatioTestStatistic
-    #result$lRatioTestPValue = pValue
+  # if (!is.null(restriction_matrix)) {
+  #   restrictions <- length(restriction_matrix[!is.na(restriction_matrix)])
+  # } else {
+  #   restrictions <- 0
+  #   restriction_matrix <- matrix(NA, k, k)
+  # }
+
+  Regime1 <- which(SB == 0) - 1
+  Regime2 <- which(SB == 1) - 1
+
+  best_estimation <- IdentifyVolatility(crit = crit, u = u, TB = TB, p = p, k = k, type = x$type,
+                                       Regime1 = Regime1, Regime2 = Regime2,
+                                       RestrictionMatrix = restriction_matrix, restrictions = restrictions,
+                                       Tob = Tob, SigmaHat1 = Sigma_hat1, SigmaHat2 = Sigma_hat2, Zt = Z_t, y = y,
+                                       maxIter = max.iter)
+  # Adding normalizing constant
+  best_estimation$Lik <- -(Tob * (k / 2) * log(2 * pi) + best_estimation$Lik)
+
+  if(restrictions > 0 ){
+
+
+    unrestricted_estimation <- IdentifyVolatility(crit = crit, u = u, TB = TB, p = p, k = k, type = x$type,
+                                                  Regime1 = Regime1, Regime2 = Regime2,
+                                                  RestrictionMatrix = matrix(NA, k, k), restrictions = 0,
+                                                  Tob = Tob, SigmaHat1 = Sigma_hat1, SigmaHat2 = Sigma_hat2, Zt = Z_t, y = y,
+                                                  maxIter = max.iter)
+    # Adding normalizing constant
+    unrestricted_estimation$Lik <- -(Tob*(k/2)*log(2*pi) + unrestricted_estimation$Lik)
+
+    lRatioTestStatistic = 2 * (unrestricted_estimation$Lik - best_estimation$Lik)
+    restrictions <- length(restriction_matrix[!is.na(restriction_matrix)])
+    pValue = round(1 - pchisq(lRatioTestStatistic, restrictions), 4)
     lRatioTest <- data.frame(testStatistic = lRatioTestStatistic, p.value = pValue)
     rownames(lRatioTest) <- ""
     colnames(lRatioTest) <- c("Test statistic", "p-value")
-    result$lRatioTest <- lRatioTest
   }else{
-    restriction_matrix <- NULL
-    result <- identifyVolatility(x, SB, Tob = Tob, u = u, k = k, y = y, restriction_matrix = restriction_matrix,
-                                 Sigma_hat1 = Sigma_hat1, Sigma_hat2 = Sigma_hat2, p = p, TB = TB, SBcharacter,
-                                 max.iter = max.iter, crit = crit, yOut = yOut, type = type)
+    lRatioTest <- NULL
   }
 
-  if(is.null(result$A_hat)){
+
+  #result$restriction_matrix = rmOut
+
+  if(is.null(best_estimation$A_hat)){
     if(inherits(x, "varest")){
       p <- x$p
       y <- t(x$y)
@@ -244,6 +298,34 @@ if(!is.numeric(SB)){
     }
   }
 
+  wald <- wald.test(best_estimation$Lambda, best_estimation$Fish, restrictions)
+  rownames(best_estimation$B) <- colnames(u)
+  rownames(best_estimation$Lambda) <- colnames(u)
+  rownames(best_estimation$Lambda_SE) <- colnames(u)
+  rownames(best_estimation$B_SE) <- colnames(u)
+
+  result <- list(
+    Lambda = best_estimation$Lambda,    # estimated Lambda matrix (unconditional heteroscedasticity)
+    Lambda_SE = best_estimation$Lambda_SE,  # standard errors of Lambda matrix
+    B = best_estimation$B,              # estimated B matrix (unique decomposition of the covariance matrix)
+    B_SE = best_estimation$B_SE,            # standard errors of B matrix
+    n = Tob,                # number of observations
+    Fish = best_estimation$Fish,            # observerd fisher information matrix
+    Lik = best_estimation$Lik,             # function value of likelihood
+    wald_statistic = wald,  # results of wald test
+    iteration = best_estimation$iteration,     # number of gls estimations
+    method = "Changes in Volatility",
+    SB = SB_out,                # Structural Break in number format
+    A_hat = best_estimation$A_hat,            # VAR parameter estimated with gls
+    type = type,          # type of the VAR model e.g 'const'
+    SBcharacter = SBcharacter,             # Structural Break in input character format
+    restrictions = restrictions, # number of restrictions
+    restriction_matrix = rmOut,
+    y = yOut,                # Data
+    p = unname(p),                # number of lags
+    K = k,# number of time series
+    lRatioTest = lRatioTest
+  )
   class(result) <- "svars"
  return(result)
 }
