@@ -7,25 +7,25 @@
 #' The post-break covariance corresponds to \eqn{\Sigma_2=B\Lambda B'} where \eqn{\Lambda} is the estimated heteroskedasticity matrix.
 #'
 #' @param x An object of class 'vars', 'vec2var', 'nlVar'. Estimated VAR object
-#' @param c_lower Integer. Starting point for the algorithm to start searching for the volatility shift.
+#' @param c_lower Numeric. Starting point for the algorithm to start searching for the volatility shift.
 #'                Default is 0.3*(Total number of observations)
-#' @param c_upper Integer. Ending point for the algorithm to stop searching for the volatility shift.
-#'                Default is 0.7*(Total number of observations)
-#' @param c_step Integer. Step width of c. Default is 5
-#' @param c_fix Integer. If the transition point is known, it can be passed as an argument
+#' @param c_upper Numeric. Ending point for the algorithm to stop searching for the volatility shift.
+#'                Default is 0.7*(Total number of observations). Note that in case of a stochastic transition variable, the input requires an absolute value
+#' @param c_step Integer. Step width of c. Default is 5. Note that in case of a stochastic transition variable, the input requires an absolute value
+#' @param c_fix Numeric. If the transition point is known, it can be passed as an argument
 #'              where transition point = Number of observations - c_fix
 #' @param transition_variable A numeric vector that represents the transition variable. By default (NULL), the time is used
 #'                           as transition variable. Note that c_lower,c_upper, c_step and/or c_fix have to be adjusted
 #'                           to the specified transition variable
-#' @param gamma_lower Integer. Lower bound for gamma. Small values indicate a flat transition function. Default is -3
-#' @param gamma_upper Integer. Upper bound for gamma. Large values indicate a steep transition function. Default is 2
-#' @param gamma_step Integer. Step width of gamma. Default is 0.5
-#' @param gamma_fix Integer. A fixed value for gamma, alternative to gamma found by the function
+#' @param gamma_lower Numeric. Lower bound for gamma. Small values indicate a flat transition function. Default is -3
+#' @param gamma_upper Numeric. Upper bound for gamma. Large values indicate a steep transition function. Default is 2
+#' @param gamma_step Numeric. Step width of gamma. Default is 0.5
+#' @param gamma_fix Numeric. A fixed value for gamma, alternative to gamma found by the function
 #' @param nc Integer. Number of processor cores
 #'           Note that the smooth transition model is computationally extremely demanding.
 #' @param max.iter Integer. Number of maximum GLS iterations
-#' @param crit Integer. Critical value for the precision of the GLS estimation
-#' @param restriction_matrix Matrix. A matrix containing presupposed entries for matrix B, NA if no restriction is imposed (entries to be estimated)
+#' @param crit Numeric. Critical value for the precision of the GLS estimation
+#' @param restriction_matrix Matrix or vector. A matrix containing presupposed entries for matrix B, NA if no restriction is imposed (entries to be estimated). Alternatively, a K^2*J (or J*K^2) matrix can be passed, where J is the number of restrictions and K the number of time series. (as suggested in Luetkepohl, 2017, section 5.2.1).
 #' @param lr_test Logical. Indicates whether the restricted model should be tested against the unrestricted model via a likelihood ratio test
 #' @return A list of class "svars" with elements
 #' \item{Lambda}{Estimated heteroscedasticity matrix \eqn{\Lambda}}
@@ -43,7 +43,7 @@
 #' \item{transition_variable}{Vector of transition variable}
 #' \item{comb}{Number of all grid combinations of gamma and c}
 #' \item{transition_function}{Vector of transition function}
-#' \item{A_hat}{Estimated VAR paramter via GLS}
+#' \item{A_hat}{Estimated VAR parameter via GLS}
 #' \item{type}{Type of the VAR model e.g., 'const'}
 #' \item{y}{Data matrix}
 #' \item{p}{Number of lags}
@@ -91,12 +91,17 @@
 
 id.st <- function(x, c_lower = 0.3, c_upper = 0.7, c_step = 5, c_fix = NULL, transition_variable = NULL,
                   gamma_lower = -3, gamma_upper = 2, gamma_step = 0.5, gamma_fix = NULL, nc = 1,
-                  max.iter = 5, crit = 0.01, restriction_matrix = NULL, lr_test = FALSE){
+                  max.iter = 5, crit = 0.001, restriction_matrix = NULL, lr_test = FALSE){
 
   # Gathering information from reduced form model
   u <- Tob <- p <- k <- residY <- coef_x <- yOut <- type <- y <-  NULL
   get_var_objects(x)
 
+  # preparing restriction amtrix format and get no of restrictions
+  rmOut = restriction_matrix
+  restriction_matrix = get_restriction_matrix(restriction_matrix, k)
+  restrictions <- length(restriction_matrix[!is.na(restriction_matrix)])
+
   # Transition function
   transition_f <- function(gamma, cc, st){
     G <- (1 + exp(-exp(gamma)*(st - cc)))^(-1)
@@ -107,14 +112,16 @@ id.st <- function(x, c_lower = 0.3, c_upper = 0.7, c_step = 5, c_fix = NULL, tra
   if(is.null(gamma_fix) &  is.null(c_fix)){
     # Creating grid for iterative procedure with gamma and break point unknown
     gamma_grid <-  seq(gamma_lower, gamma_upper, by = gamma_step)
-    cc_grid <- seq(ceiling(c_lower*Tob), floor(c_upper*Tob), by = c_step)
-    grid_comb <- unique(expand.grid(gamma_grid, cc_grid))
     if(is.null(transition_variable)){
+      cc_grid <- seq(ceiling(c_lower*Tob), floor(c_upper*Tob), by = c_step)
+      grid_comb <- unique(expand.grid(gamma_grid, cc_grid))
       G_grid <- mapply(transition_f, grid_comb[,1], grid_comb[,2], MoreArgs = list(st = seq(1:Tob)))
     }else{
       if(length(transition_variable) != Tob){
         stop('length of transition variable is unequal to data length')
       }
+      cc_grid <- seq(c_lower, c_upper, by = c_step)
+      grid_comb <- unique(expand.grid(gamma_grid, cc_grid))
       G_grid <- mapply(transition_f, grid_comb[,1], grid_comb[,2], MoreArgs = list(st = transition_variable))
     }
   }else if(is.null(gamma_fix)){
@@ -124,14 +131,17 @@ id.st <- function(x, c_lower = 0.3, c_upper = 0.7, c_step = 5, c_fix = NULL, tra
     G_grid <- mapply(transition_f, grid_comb[,1], grid_comb[,2], MoreArgs = list(st = seq(1:Tob)))
   }else if(is.null(c_fix)){
     # Creating grid for iterative procedure with fix shape of transition function
-    cc_grid <- seq(ceiling(c_lower*Tob), floor(c_upper*Tob), by = c_step)
-    grid_comb <- unique(expand.grid(gamma_fix, cc_grid))
+
     if(is.null(transition_variable)){
+      cc_grid <- seq(ceiling(c_lower*Tob), floor(c_upper*Tob), by = c_step)
+      grid_comb <- unique(expand.grid(gamma_fix, cc_grid))
       G_grid <- mapply(transition_f, grid_comb[,1], grid_comb[,2], MoreArgs = list(st = seq(1:Tob)))
     }else{
       if(length(transition_variable) != Tob){
         stop('length of transition variable is unequal to data length')
       }
+      cc_grid <- seq(c_lower, c_upper, by = c_step)
+      grid_comb <- unique(expand.grid(gamma_fix, cc_grid))
       G_grid <- mapply(transition_f, grid_comb[,1], grid_comb[,2], MoreArgs = list(st = transition_variable))
     }
   }else{
@@ -146,34 +156,44 @@ id.st <- function(x, c_lower = 0.3, c_upper = 0.7, c_step = 5, c_fix = NULL, tra
     }
   }
 
-  yl <- t(y_lag_cr(t(y), p)$lags)
+  #ylold <- t(y_lag_cr(t(y), p)$lags)
+  yl <- t(YLagCr(t(y), p))
   #yret <- y
   y_loop <- y[,-c(1:p)]
 
   if(x$type == 'const'){
     Z_t <- rbind(rep(1, ncol(yl)), yl)
   }else if(x$type == 'trend'){
-    Z_t <- rbind(seq(p + 1, Tob), yl)
+    Z_t <- rbind(seq(p + 1, Tob + p), yl)
   }else if(x$type == 'both'){
-    Z_t <- rbind(rep(1, ncol(yl)), seq(p + 1, Tob), yl)
+    Z_t <- rbind(rep(1, ncol(yl)), seq(p + 1, Tob + p), yl)
   }else{
     Z_t <- yl
   }
 
   if(!is.null(gamma_fix) &  !is.null(c_fix)){
-    best_estimation <- iterative_smooth_transition(transition = G_grid, u = u, y = y, Tob = Tob, k = k,
-                                           p = p, crit = crit, max.iter = max.iter, Z_t = Z_t, y_loop = y_loop,
-                                           restriction_matrix = restriction_matrix)
+    if (!is.null(restriction_matrix)) {
+      restrictions <- length(restriction_matrix[!is.na(restriction_matrix)])
+    } else {
+      restrictions <- 0
+      restriction_matrix <- matrix(NA, k, k)
+    }
+
+    best_estimation <- IterativeSmoothTransition(transition = G_grid, u = u, Tob = Tob, k = k, p = p,
+                                                 crit = crit, maxIter = max.iter, Z_t = Z_t, Yloop = y_loop,
+                                                 RestrictionMatrix = restriction_matrix, restrictions = restrictions)
     transition_function <- G_grid
 
     transition_coefficient <- gamma_fix
     SB <- c_fix
     comb <- 1
 
-    if(lr_test == TRUE & !is.null(restriction_matrix)){
-      unrestricted_estimation <- iterative_smooth_transition(G_grid, u = u, y = y, Tob = Tob, k = k,
-                                                             p = p, crit = crit, max.iter = max.iter, Z_t = Z_t, y_loop = y_loop,
-                                                             restriction_matrix = NULL)
+    if(lr_test == TRUE & restrictions > 0){
+
+      unrestricted_estimation <- IterativeSmoothTransition(transition = G_grid, u = u, Tob = Tob, k = k, p = p,
+                                                           crit = crit, maxIter = max.iter, Z_t = Z_t, Yloop = y_loop,
+                                                           RestrictionMatrix = matrix(NA, k, k), restrictions = 0)
+
       lRatioTestStatistic = 2 * (unrestricted_estimation$Lik - best_estimation$Lik)
       restrictions <- length(restriction_matrix[!is.na(restriction_matrix)])
       pValue = round(1 - pchisq(lRatioTestStatistic, restrictions), 4)
@@ -185,15 +205,15 @@ id.st <- function(x, c_lower = 0.3, c_upper = 0.7, c_step = 5, c_fix = NULL, tra
     }
 
   }else{
+
     G_grid <- apply(G_grid, 2, list)
 
-    grid_optimization <- pblapply(G_grid, function(x){iterative_smooth_transition(unlist(x),
-                                                                                  u = u, y = y,
-                                                                                  Tob = Tob, k = k,
-                                                                                  p = p, crit = crit,
-                                                                                  max.iter = max.iter, Z_t = Z_t,
-                                                                                  y_loop = y_loop,
-                                                                                  restriction_matrix = restriction_matrix)},
+    grid_optimization <- pblapply(G_grid, function(x){IterativeSmoothTransition(unlist(x),
+                                                                                  u = u, Tob = Tob, k = k,
+                                                                                  p = p, crit = crit, Yloop = y_loop,
+                                                                                  maxIter = max.iter, Z_t = Z_t,
+                                                                                  RestrictionMatrix = restriction_matrix,
+                                                                                  restrictions = restrictions)},
                                   cl = nc)
 
     max_likelihood <- which.max(sapply(grid_optimization, '[[', 'Lik'))
@@ -215,9 +235,9 @@ id.st <- function(x, c_lower = 0.3, c_upper = 0.7, c_step = 5, c_fix = NULL, tra
         G_grid <- mapply(transition_f, grid_comb[,1], grid_comb[,2], MoreArgs = list(st = transition_variable))
       }
 
-      unrestricted_estimation <- iterative_smooth_transition(G_grid, u = u, y = y, Tob = Tob, k = k,
-                                                             p = p, crit = crit, max.iter = max.iter, Z_t = Z_t, y_loop = y_loop,
-                                                             restriction_matrix = NULL)
+      unrestricted_estimation <- IterativeSmoothTransition(G_grid, u = u, Tob = Tob, k = k,
+                                                             p = p, crit = crit, maxIter = max.iter, Z_t = Z_t, Yloop = y_loop,
+                                                             RestrictionMatrix = matrix(NA, k, k), restrictions = 0)
       lRatioTestStatistic = 2 * (unrestricted_estimation$Lik - best_estimation$Lik)
       restrictions <- length(restriction_matrix[!is.na(restriction_matrix)])
       pValue = round(1 - pchisq(lRatioTestStatistic, restrictions), 4)
@@ -229,11 +249,6 @@ id.st <- function(x, c_lower = 0.3, c_upper = 0.7, c_step = 5, c_fix = NULL, tra
     }
   }
 
-  if(!is.null(restriction_matrix)){
-    restrictions <- length(restriction_matrix[!is.na(restriction_matrix)])
-  }else{
-    restrictions <- 0
-  }
 
   # Testing the estimated SVAR for identification by means of wald statistic
   wald <- wald.test(best_estimation$Lambda, best_estimation$Fish, restrictions)
@@ -264,7 +279,7 @@ id.st <- function(x, c_lower = 0.3, c_upper = 0.7, c_step = 5, c_fix = NULL, tra
     p = p,                # number of lags
     K = k,                 # number of time series
     restrictions = restrictions,
-    restriction_matrix = restriction_matrix,
+    restriction_matrix = rmOut,
     lr_test = lr_test,
     lRatioTest = lRatioTest
   )
